from typing import Any

from django.core.management import BaseCommand

class Command(BaseCommand):
    shells: list[str]
    def ipython(self, options: Any) -> None: ...
    def bpython(self, options: Any) -> None: ...
    def python(self, options: Any) -> None: ...
