import sys
from typing import Any

class Str: ...

class Xxo:
    def demo(self) -> None: ...

def foo(__i: int, __j: int) -> Any: ...
def new() -> Xxo: ...

if sys.version_info >= (3, 10):
    class Error: ...

else:
    class error: ...
    class Null: ...
    def roj(__b: Any) -> None: ...
