from multiprocessing import AuthenticationError as AuthenticationError
from multiprocessing import BufferTooShort as BufferTooShort
from multiprocessing import ProcessError as ProcessError
from multiprocessing import TimeoutError as TimeoutError

class TimeLimitExceeded(Exception): ...
class SoftTimeLimitExceeded(Exception): ...
class WorkerLostError(Exception): ...
class Terminated(Exception): ...
class RestartFreqExceeded(Exception): ...
class CoroStop(Exception): ...
