from nwon_django_toolbox.tests.api_method_tests.check_delete import (
    check_delete_basics,
    check_delete_not_allowed,
)
from nwon_django_toolbox.tests.api_method_tests.check_get import check_get_basics
from nwon_django_toolbox.tests.api_method_tests.check_patch import (
    check_patch_basics,
    check_patch_method_not_allowed,
    check_patch_parameters_failing,
    check_patch_parameters_successful,
    check_patch_read_only_field,
)
from nwon_django_toolbox.tests.api_method_tests.check_permissions import (
    check_delete_permissions,
    check_get_detail_permissions,
    check_get_list_permissions,
    check_patch_permissions,
    check_permission,
    check_permissions,
    check_post_permissions,
    check_put_permissions,
)
from nwon_django_toolbox.tests.api_method_tests.check_post import (
    check_post_basics,
    check_post_not_allowed,
    check_post_parameters_failing,
    check_post_parameters_not_required,
    check_post_parameters_required,
    check_post_parameters_successful,
    check_post_read_only_field,
)
from nwon_django_toolbox.tests.api_method_tests.check_put import (
    check_put_basics,
    check_put_not_allowed,
    check_put_parameters_failing,
    check_put_parameters_successful,
    check_put_read_only_field,
)
from nwon_django_toolbox.tests.helper.ensure_key_with_object_list import (
    ensure_key_with_object_list,
)
from nwon_django_toolbox.tests.helper.ensure_paged_results import ensure_paged_results

__all__ = [
    "check_delete_basics",
    "check_delete_not_allowed",
    "check_get_basics",
    "check_patch_basics",
    "check_patch_method_not_allowed",
    "check_patch_parameters_failing",
    "check_patch_parameters_successful",
    "check_patch_read_only_field",
    "check_put_basics",
    "check_put_not_allowed",
    "check_post_parameters_required",
    "check_put_parameters_failing",
    "check_put_parameters_successful",
    "check_put_read_only_field",
    "check_post_basics",
    "check_post_not_allowed",
    "check_post_parameters_failing",
    "check_post_parameters_successful",
    "check_post_read_only_field",
    "check_post_parameters_not_required",
    "ensure_key_with_object_list",
    "ensure_paged_results",
    "check_delete_permissions",
    "check_get_detail_permissions",
    "check_get_list_permissions",
    "check_patch_permissions",
    "check_permission",
    "check_permissions",
    "check_post_permissions",
    "check_put_permissions",
]
