from typing import Any

from django.http.response import HttpResponse

def compress_kml(kml: str) -> bytes: ...
def render_to_kml(*args: Any, **kwargs: Any) -> HttpResponse: ...
def render_to_kmz(*args: Any, **kwargs: Any) -> HttpResponse: ...
